@@ -33,9 +33,15 @@ class CategoryInline(admin.TabularInline):
     extra = 1
 
 
+class ProductAttributeInline(admin.TabularInline):
+    model = ProductAttribute
+    extra = 2
+
+
 class ProductClassAdmin(admin.ModelAdmin):
     list_display = ('name', 'requires_shipping', 'track_stock')
     prepopulated_fields = {"slug": ("name",)}
+    inlines = [ProductAttributeInline]
 
 
 class ContributorAdmin(admin.ModelAdmin):
@@ -48,13 +54,14 @@ class ContributorRoleAdmin(admin.ModelAdmin):
 
 class ProductAdmin(admin.ModelAdmin):
     list_display = ('get_title', 'upc', 'get_product_class', 'is_top_level',
-                    'is_group', 'is_variant', 'attribute_summary',
+                    'is_variant', 'attribute_summary',
                     'date_created')
     prepopulated_fields = {"slug": ("title",)}
     inlines = [AttributeInline, CategoryInline, ProductRecommendationInline]
 
 
 class ProductAttributeAdmin(admin.ModelAdmin):
+    list_display = ('name', 'code', 'product_class', 'type')
     prepopulated_fields = {"code": ("name", )}
 
 
@@ -71,6 +78,7 @@ class AttributeOptionInline(admin.TabularInline):
 
 
 class AttributeOptionGroupAdmin(admin.ModelAdmin):
+    list_display = ('name', 'option_summary')
     inlines = [AttributeOptionInline, ]
 
 
