@@ -1,5 +1,7 @@
+import warnings
 from django.conf import settings
-from django.http import HttpResponsePermanentRedirect, Http404
+from django.http import HttpResponsePermanentRedirect
+from django.shortcuts import get_object_or_404
 from django.views.generic import ListView, DetailView
 from django.db.models import get_model
 from django.utils.translation import ugettext_lazy as _
@@ -11,8 +13,7 @@ Product = get_model('catalogue', 'product')
 ProductReview = get_model('reviews', 'ProductReview')
 Category = get_model('catalogue', 'category')
 ProductAlert = get_model('customer', 'ProductAlert')
-ProductAlertForm = get_class('customer.forms',
-                             'ProductAlertForm')
+ProductAlertForm = get_class('customer.forms', 'ProductAlertForm')
 
 
 class ProductDetailView(DetailView):
@@ -21,18 +22,37 @@ class ProductDetailView(DetailView):
     view_signal = product_viewed
     template_folder = "catalogue"
 
-    def get_object(self):
-        # Use a cached version to prevent unnecessary DB calls
-        if not hasattr(self, '_product'):
-            setattr(self, '_product',
-                    super(ProductDetailView, self).get_object())
-        return self._product
+    def get(self, request, **kwargs):
+        """
+        Ensures that the correct URL is used before rendering a response
+        """
+        self.object = product = self.get_object()
+
+        if product.is_variant:
+            return HttpResponsePermanentRedirect(
+                product.parent.get_absolute_url())
+
+        correct_path = product.get_absolute_url()
+        if correct_path != request.path:
+            return HttpResponsePermanentRedirect(correct_path)
+
+        response = super(ProductDetailView, self).get(request, **kwargs)
+        self.send_signal(request, response, product)
+        return response
+
+    def get_object(self, queryset=None):
+        # Check if self.object is already set to prevent unnecessary DB calls
+        if hasattr(self, 'object'):
+            return self.object
+        else:
+            return super(ProductDetailView, self).get_object(queryset)
 
     def get_context_data(self, **kwargs):
         ctx = super(ProductDetailView, self).get_context_data(**kwargs)
         ctx['reviews'] = self.get_reviews()
         ctx['alert_form'] = self.get_alert_form()
         ctx['has_active_alert'] = self.get_alert_status()
+
         return ctx
 
     def get_alert_status(self):
@@ -42,76 +62,76 @@ class ProductDetailView(DetailView):
             alerts = ProductAlert.objects.filter(
                 product=self.object, user=self.request.user,
                 status=ProductAlert.ACTIVE)
-            has_alert = alerts.count() > 0
+            has_alert = alerts.exists()
         return has_alert
 
     def get_alert_form(self):
-        return ProductAlertForm(user=self.request.user,
-                                product=self.object)
+        return ProductAlertForm(
+            user=self.request.user, product=self.object)
 
     def get_reviews(self):
         return self.object.reviews.filter(status=ProductReview.APPROVED)
 
-    def get(self, request, **kwargs):
-        # Ensure that the correct URL is used
-        product = self.get_object()
-        correct_path = product.get_absolute_url()
-        if correct_path != request.path:
-            return HttpResponsePermanentRedirect(correct_path)
-
-        response = super(ProductDetailView, self).get(request, **kwargs)
-        self.send_signal(request, response, product)
-        return response
-
     def send_signal(self, request, response, product):
-        self.view_signal.send(sender=self, product=product, user=request.user,
-                              request=request, response=response)
+        self.view_signal.send(
+            sender=self, product=product, user=request.user, request=request,
+            response=response)
 
     def get_template_names(self):
         """
         Return a list of possible templates.
 
         We try 2 options before defaulting to catalogue/detail.html:
-        1). detail-for-upc-<upc>.html
-        2). detail-for-class-<classname>.html
+            1). detail-for-upc-<upc>.html
+            2). detail-for-class-<classname>.html
 
         This allows alternative templates to be provided for a per-product
         and a per-item-class basis.
         """
-        product = self.get_object()
-        names = ['%s/detail-for-upc-%s.html' % (self.template_folder, product.upc),
-                 '%s/detail-for-class-%s.html' % (self.template_folder, product.product_class.name.lower()),
-                 '%s/detail.html' % (self.template_folder)]
-        return names
+        return [
+            '%s/detail-for-upc-%s.html' % (
+                self.template_folder, self.object.upc),
+            '%s/detail-for-class-%s.html' % (
+                self.template_folder, self.object.get_product_class().slug),
+            '%s/detail.html' % (self.template_folder)]
 
 
 def get_product_base_queryset():
     """
-    Return ``QuerySet`` for product model with related
-    content pre-loaded. The ``QuerySet`` returns unfiltered
-    results for further filtering.
+    Deprecated. Kept only for backwards compatibility.
+    Product.browsable.base_queryset() should be used instead.
     """
-    return Product.browsable.select_related(
-        'product_class',
-    ).prefetch_related(
-        'reviews',
-        'variants',
-        'product_options',
-        'product_class__options',
-        'stockrecord',
-        'images',
-    ).all()
+    warnings.warn(("`get_product_base_queryset` is deprecated in favour of"
+                   "`base_queryset` on Product's managers. It will be removed"
+                   "in Oscar 0.7."), DeprecationWarning)
+    return Product.browsable.base_queryset()
 
 
 class ProductCategoryView(ListView):
     """
     Browse products in a given category
+
+    Category URLs used to be based on solely the slug. Renaming the category
+    or any of the parent categories would break the URL. Hence, the new URLs
+    consist of both the slug and category PK (compare product URLs).
+    The legacy way still works to not break existing systems.
     """
     context_object_name = "products"
     template_name = 'catalogue/browse.html'
     paginate_by = settings.OSCAR_PRODUCTS_PER_PAGE
 
+    def get_object(self):
+        if 'pk' in self.kwargs:
+            self.category = get_object_or_404(Category, pk=self.kwargs['pk'])
+        else:
+            self.category = get_object_or_404(
+                Category, slug=self.kwargs['category_slug'])
+
     def get(self, request, *args, **kwargs):
+        self.get_object()
+        correct_path = self.category.get_absolute_url()
+        if correct_path != request.path:
+            return HttpResponsePermanentRedirect(correct_path)
         self.categories = self.get_categories()
         return super(ProductCategoryView, self).get(request, *args, **kwargs)
 
@@ -119,25 +139,20 @@ class ProductCategoryView(ListView):
         """
         Return a list of the current category and it's ancestors
         """
-        slug = self.kwargs['category_slug']
-        try:
-            category = Category.objects.get(slug=slug)
-        except Category.DoesNotExist:
-            raise Http404()
-        categories = list(category.get_descendants())
-        categories.append(category)
+        categories = list(self.category.get_descendants())
+        categories.append(self.category)
         return categories
 
     def get_context_data(self, **kwargs):
         context = super(ProductCategoryView, self).get_context_data(**kwargs)
 
         context['categories'] = self.categories
-        context['category'] = self.categories[-1]
-        context['summary'] = self.categories[-1].name
+        context['category'] = self.category
+        context['summary'] = self.category.name
         return context
 
     def get_queryset(self):
-        return get_product_base_queryset().filter(
+        return Product.browsable.base_queryset().filter(
             categories__in=self.categories
         ).distinct()
 
@@ -158,12 +173,14 @@ class ProductListView(ListView):
 
     def get_queryset(self):
         q = self.get_search_query()
+        qs = Product.browsable.base_queryset()
         if q:
-            # Send signal to record the view of this product
-            self.search_signal.send(sender=self, query=q, user=self.request.user)
-            return get_product_base_queryset().filter(title__icontains=q)
+            # Send signal to record this search
+            self.search_signal.send(sender=self, query=q,
+                                    user=self.request.user)
+            return qs.filter(title__icontains=q)
         else:
-            return get_product_base_queryset()
+            return qs
 
     def get_context_data(self, **kwargs):
         context = super(ProductListView, self).get_context_data(**kwargs)
@@ -171,6 +188,7 @@ class ProductListView(ListView):
         if not q:
             context['summary'] = _('All products')
         else:
-            context['summary'] = _("Products matching '%(query)s'") % {'query': q}
+            context['summary'] = _("Products matching '%(query)s'") \
+                % {'query': q}
             context['search_term'] = q
         return context
