@@ -1,5 +1,4 @@
 from django.utils.timezone import now
-
 from django.db import models
 
 
@@ -12,3 +11,12 @@ class ActiveOfferManager(models.Manager):
         return super(ActiveOfferManager, self).get_query_set().filter(
             models.Q(end_datetime__gte=cutoff) | models.Q(end_datetime=None),
             start_datetime__lte=cutoff)
+
+
+class BrowsableRangeManager(models.Manager):
+    """
+    For searching only ranges which have the "is_browsable" flag set to True.
+    """
+    def get_query_set(self):
+        return super(BrowsableRangeManager, self).get_query_set().filter(
+            is_public=True)
