@@ -24,15 +24,17 @@ class AbstractOrder(models.Model):
     basket_id = models.PositiveIntegerField(
         _("Basket ID"), null=True, blank=True)
 
-    # Orders can be anonymous so we don't always have a customer ID
+    # Orders can be placed without the user authenticating so we don't always
+    # have a customer ID.
     user = models.ForeignKey(
         AUTH_USER_MODEL, related_name='orders', null=True, blank=True,
-        verbose_name=_("User"))
+        verbose_name=_("User"), on_delete=models.SET_NULL)
 
     # Billing address is not always required (eg paying by gift card)
     billing_address = models.ForeignKey(
         'order.BillingAddress', null=True, blank=True,
-        verbose_name=_("Billing Address"))
+        verbose_name=_("Billing Address"),
+        on_delete=models.SET_NULL)
 
     # Total price looks like it could be calculated by adding up the
     # prices of the associated lines, but in some circumstances extra
@@ -56,7 +58,8 @@ class AbstractOrder(models.Model):
     # address is not mandatory.
     shipping_address = models.ForeignKey(
         'order.ShippingAddress', null=True, blank=True,
-        verbose_name=_("Shipping Address"))
+        verbose_name=_("Shipping Address"),
+        on_delete=models.SET_NULL)
     shipping_method = models.CharField(
         _("Shipping method"), max_length=128, null=True, blank=True)
 
