@@ -21,14 +21,17 @@ class DefaultWrapper(object):
             return True
         return stockrecord.net_stock_level > 0
 
-    def is_purchase_permitted(self, stockrecord, user=None, quantity=1):
+    def is_purchase_permitted(self, stockrecord, user=None, quantity=1,
+                              product=None):
         """
         Test whether a particular purchase is possible (is a user buying a
         given quantity of the product)
         """
+        # check, if fetched product is provided, to avoid db call
+        product = product or stockrecord.product
         if not self.is_available_to_buy(stockrecord):
-            return False, _("'%s' is not available to purchase") % stockrecord.product.title
-        max_qty = self.max_purchase_quantity(stockrecord, user)
+            return False, _("'%s' is unavailable to purchase") % product.title
+        max_qty = self.max_purchase_quantity(stockrecord, user, product)
         if max_qty is None:
             return True, None
         if max_qty < quantity:
@@ -36,11 +39,13 @@ class DefaultWrapper(object):
                             {'max': max_qty})
         return True, None
 
-    def max_purchase_quantity(self, stockrecord, user=None):
+    def max_purchase_quantity(self, stockrecord, user=None, product=None):
         """
         Return the maximum available purchase quantity for a given user
         """
-        if not stockrecord.product.product_class.track_stock:
+        product = product or stockrecord.product
+
+        if not product.get_product_class().track_stock:
             return None
         if stockrecord.num_in_stock is None:
             return None
@@ -52,7 +57,8 @@ class DefaultWrapper(object):
 
         This is normally used within CSS to add icons to stock messages
 
-        :param oscar.apps.partner.models.StockRecord stockrecord: stockrecord instance
+        :param oscar.apps.partner.models.StockRecord stockrecord: stockrecord
+        instance
         """
         if stockrecord.net_stock_level > 0:
             return self.CODE_IN_STOCK
@@ -64,7 +70,8 @@ class DefaultWrapper(object):
         """
         Return an availability message for the passed stockrecord.
 
-        :param oscar.apps.partner.models.StockRecord stockrecord: stockrecord instance
+        :param oscar.apps.partner.models.StockRecord stockrecord: stockrecord
+        instance
         """
         if stockrecord.net_stock_level > 0:
             return _("In stock (%d available)") % stockrecord.net_stock_level
